@@ -4,8 +4,8 @@
 ;; Author: <github.com/Anoncheg1,codeberg.org/Anoncheg>
 ;; Keywords: org, comm, url, link
 ;; URL: https://github.com/Anoncheg1/emacs-oai
-;; Package-Version: 20260109.915
-;; Package-Revision: 1cc12a88a140
+;; Package-Version: 20260121.903
+;; Package-Revision: bd09b6b1bae2
 ;; Created: 27 dec 2025
 ;; Package-Requires: ((emacs "29.1"))
 ;; Optional dependency: ((org-links "0.2"))
@@ -108,6 +108,12 @@
 ;; - key to enable full Org highlighting? think about it
 ;; - fontify latex [[file:/usr/share/emacs/30.2/lisp/org/org.el::16097::(defun org-inside-latex-macro-p ()]]
 ;; [[file:/usr/share/emacs/30.2/lisp/textmodes/tex-mode.el::1277::(setq-local font-lock-defaults]]
+;; - add support for one line code blocks: ```toon users[3]{id,name,role}: 1,Alice,admin 2,Bob,user 3,Charlie,user
+;; - small markdown mode on highlighting
+;; - simple Elisp function to ask LLM
+;; - add guide to use `oai-restapi-request' and with retries for simple ELisp LLM call and get result
+;; - add option for tag to expand only the last user prompt or in all.
+;; - C-c C-k should jump to current begining of message, not next
 ;;; Code:
 ;; Touch: Pain, water and warm.
 
@@ -263,7 +269,7 @@ messages."
                                 (expanded (oai-block--stringify-chat-messages expanded)))
                            expanded))))
     (if (called-interactively-p 'any)
-        (let ((buf (get-buffer-create "*OAi Preview*")))
+        (let ((buf (get-buffer-create "*OAI Preview*")))
           (with-help-window buf (with-current-buffer buf
                                   (insert res-str)))
           (switch-to-buffer buf))
